// base URL when deploying the frontend app
export const BASE_URL = import.meta.env.BASE_URL;
// backend API URL, used by the client to make API requests
export const BACKEND_API = import.meta.env.VITE_BACKEND_API;
